/*#############################################################
Driver name	    : LM6029ACW_F3.c
Author 					: Grant Phillips
Date Modified   : 25/08/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)
Tested On       : STM32F3-Discovery

Description			: Provides a library to control the position 
									of up to four (0 - 3) servos.

Requirements    : * STM32F3-Discovery Board
									* Define the SET and RESET functions for the
										output port pins.
									* Do the proper GPIO initialization for the
										output port pins in LM6029ACW_Init();

Functions				: LM6029ACW_Init
									LM6029ACW_ClrScr
									LM6029ACW_PowerSave
									LM6029ACW_PowerActive
									LM6029ACW_DrawPixel
									LM6029ACW_DrawLine
									LM6029ACW_DrawCircle
									LM6029ACW_GotoXY
									LM6029ACW_PutStr
									LM6029ACW_PutChar
													  
Special Note(s) : NONE
##############################################################*/

#include <stdio.h>
#include <math.h>

/* //////////////////////////////////////////////////////////////
Define the SET and RESET functions for each of the output port 
pins for the LM6029ACW here.  
Also do the proper GPIO initialization for these port pins in
LM6029ACW_Init().
////////////////////////////////////////////////////////////// */
#define lcd_cs_1		GPIO_WriteBit(GPIOA, GPIO_Pin_8, Bit_SET)			//CS = 1
#define lcd_cs_0		GPIO_WriteBit(GPIOA, GPIO_Pin_8, Bit_RESET)		//CS = 0
#define lcd_rst_1		GPIO_WriteBit(GPIOA, GPIO_Pin_4, Bit_SET)			//RST = 1
#define lcd_rst_0		GPIO_WriteBit(GPIOA, GPIO_Pin_4, Bit_RESET)		//RST = 0
#define lcd_rs_1		GPIO_WriteBit(GPIOA, GPIO_Pin_3, Bit_SET)			//RS = 1
#define lcd_rs_0		GPIO_WriteBit(GPIOA, GPIO_Pin_3, Bit_RESET)		//RS = 0
#define lcd_wr_1		GPIO_WriteBit(GPIOA, GPIO_Pin_2, Bit_SET)			//WR = 1
#define lcd_wr_0		GPIO_WriteBit(GPIOA, GPIO_Pin_2, Bit_RESET)		//WR = 0
#define lcd_rd_1		GPIO_WriteBit(GPIOA, GPIO_Pin_1, Bit_SET)			//RD = 1
#define lcd_rd_0		GPIO_WriteBit(GPIOA, GPIO_Pin_1, Bit_RESET)		//RD = 0
//#define lcd_bla_1		GPIO_WriteBit(GPIOC, GPIO_Pin_5, Bit_SET)		//BLA = 1 (if an output is used for switching the LCD backlight on or off)
//#define lcd_bla_0		GPIO_WriteBit(GPIOC, GPIO_Pin_5, Bit_RESET)	//BLA = 0 (if an output is used for switching the LCD backlight on or off)
#define lcd_data  		GPIOD			//always assume using bit7-0 on this port
#define lcd_delayx		1					//delay for writing and reading from lcd
#define lcd_init_delay	500			//delay when initializing the lcd



void LM6029ACW_Init(void);
void LM6029ACW_Command(unsigned char cmd);
void LM6029ACW_WriteData(unsigned char dat);
unsigned char LM6029ACW_ReadData(void);
void LM6029ACW_SetPage(unsigned char page);
void LM6029ACW_SetColumn(unsigned char Col);
void LM6029ACW_ClrScr(unsigned char color);
void LM6029ACW_Delay(unsigned int del);
void LM6029ACW_PowerSave(void);
void LM6029ACW_PowerActive(void);
void LM6029ACW_DrawPixel(unsigned char x, unsigned char y, unsigned char color);
void LM6029ACW_DrawLine(int x1, int y1, int x2, int y2, unsigned int color);
void LM6029ACW_DrawCircle(unsigned char x, unsigned char y, unsigned char radius, unsigned char fill, unsigned char color);
void LM6029ACW_WriteCharCol(unsigned char v, unsigned char x, unsigned char page, unsigned char color);
void LM6029ACW_DrawChar(unsigned char c, unsigned char x, unsigned char page, unsigned char color);

void LM6029ACW_GotoXY(unsigned char x, unsigned char y);
void LM6029ACW_PutChar(unsigned char c, unsigned char color);
void LM6029ACW_PutStr(char *str, unsigned char color);

unsigned char LM6029ACW_DisplayRAM[128][8];
unsigned char lcd_char_x=0, lcd_char_y=0;



/*********************************************************************************************
Function name   : LM6029ACW_Init
Author 					: Grant Phillips
Date Modified   : 25/08/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Initializes the LM6029ACW lcd module

Special Note(s) : NONE

Parameters			: NONE
Return value		: NONE
*********************************************************************************************/
void LM6029ACW_Init(void)
{
	unsigned char i,j;
	GPIO_InitTypeDef GPIO_InitStructure;	
	
	/* //////////////////////////////////////////////////////////////
	Do the proper GPIO initialization for the output port pins here
	////////////////////////////////////////////////////////////// */
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA | RCC_AHBPeriph_GPIOD, ENABLE);

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3 | GPIO_Pin_4 | GPIO_Pin_8;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;									
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3 | GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_6 | GPIO_Pin_7;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;									
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOD, &GPIO_InitStructure);
	/////////////////////////////////////////////////////////////////
	
	lcd_rd_1;
	lcd_cs_0;
	lcd_rst_0;
	LM6029ACW_Delay(lcd_init_delay);
	lcd_rst_1;

	LM6029ACW_Command(0xa0);	//ADC=0 Normal
	LM6029ACW_Command(0xc8);	//SHL=1 flipped in y-direction
	LM6029ACW_Command(0xa2);	//Bias=0; 1/9 Bias
	LM6029ACW_Command(0x2f);	//Power Control VF, VR, VC
	LM6029ACW_Command(0xae);	//Display OFF
	LM6029ACW_Command(0x81);	//Set voltage reference mode
	LM6029ACW_Command(0x29);	//Power Control VF
	LM6029ACW_Command(0x40);	//Set display start line
	LM6029ACW_Command(0xaf);	//Display ON

	for(i=0;i<128;i++)
		for(j=0;j<8;j++)
			LM6029ACW_DisplayRAM[i][j]=0x00;
	/*
	for(i=0;i<40;i++)
		for(j=0;j<5;j++)
			map[i][j]=0x00;
	*/
	lcd_cs_1;
	
	LM6029ACW_ClrScr(0);
}



/*********************************************************************************************
Function name   : LM6029ACW_Delay
Author 					: Grant Phillips
Date Modified   : 25/08/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Creates a delay which is simply a code loop which is independent from any
									hardware timers.

Special Note(s) : NONE

Parameters			: del		-	16-bit value to represent the delay cycles
Return value		: NONE
*********************************************************************************************/
void LM6029ACW_Delay(unsigned int del)
{
	unsigned int i=0;

	while(i<del)
		i++;
}



/*********************************************************************************************
Function name   : LM6029ACW_Command
Author 					: Grant Phillips
Date Modified   : 25/08/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Writes a COMMAND to the LM6029ACW lcd module

Special Note(s) : NONE

Parameters			: cmd		-	The byte of command to write to the lcd
Return value		: NONE
*********************************************************************************************/
void LM6029ACW_Command(unsigned char cmd)
{
	lcd_rs_0;
	//lcd_rd_1;

	//GPIO_Write(lcd_data, cmd);
	GPIO_Write(lcd_data, (GPIO_ReadOutputData(lcd_data) & 0xff00) | cmd);
	LM6029ACW_Delay(lcd_delayx);
	lcd_wr_0;
	LM6029ACW_Delay(lcd_delayx);
	lcd_wr_1;
}



/*********************************************************************************************
Function name   : LM6029ACW_WriteData
Author 					: Grant Phillips
Date Modified   : 25/08/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Writes DATA to the LM6029ACW lcd module

Special Note(s) : NONE

Parameters			: dat		-	The byte of data to write to the lcd
Return value		: NONE
*********************************************************************************************/
void LM6029ACW_WriteData(unsigned char dat)
{
	lcd_rs_1;
	//lcd_rd_1;

	//GPIO_Write(lcd_data, dat);
	GPIO_Write(lcd_data, (GPIO_ReadOutputData(lcd_data) & 0xff00) | dat);
	LM6029ACW_Delay(lcd_delayx);
	lcd_wr_0;
	LM6029ACW_Delay(lcd_delayx);
	lcd_wr_1;
}



/*********************************************************************************************
Function name   : LM6029ACW_SetPage
Author 					: Grant Phillips
Date Modified   : 25/08/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Set the display RAM page address

Special Note(s) : NONE

Parameters			: Page		-	4-bit value representing the page address (0 - 7)
Return value		: NONE
*********************************************************************************************/
void LM6029ACW_SetPage(unsigned char Page)
{
	Page = Page & 0x0f;
	Page = Page | 0xb0;
	LM6029ACW_Command(Page);
}



/*********************************************************************************************
Function name   : LM6029ACW_SetColumn
Author 					: Grant Phillips
Date Modified   : 25/08/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Set the column address counter

Special Note(s) : NONE

Parameters			: Page		-	byte value representing the column (0 - 127)
Return value		: NONE
*********************************************************************************************/
void LM6029ACW_SetColumn(unsigned char Col)
{
	unsigned char temp;

	temp = Col;
	Col = Col & 0x0f;
	Col = Col | 0x00;
	LM6029ACW_Command(Col);
	temp = temp >> 4;
	Col = temp & 0x0f;
	Col = Col | 0x10;
	LM6029ACW_Command(Col);
}



/*********************************************************************************************
Function name   : LM6029ACW_ClrScr
Author 					: Grant Phillips
Date Modified   : 25/08/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Clears the display by clearing or filling all the pixels

Special Note(s) : NONE

Parameters			: color		-	bit value; clear pixels(0) or fill pixels(1)
Return value		: NONE
*********************************************************************************************/
void LM6029ACW_ClrScr(unsigned char color)
{
	unsigned char i, j, x=0;

	lcd_cs_0;
	for(i=0; i<8; i++)
	{
		x=0;
		LM6029ACW_SetColumn(0);
		LM6029ACW_SetPage(i);
		for(j=0; j<128; j++)
			if(color==0)
			{
				LM6029ACW_WriteData(0x00);
				LM6029ACW_DisplayRAM[x][i] = 0x00;
				x++;
			}
			else
			{
				LM6029ACW_WriteData(0xff);
				LM6029ACW_DisplayRAM[x][i] = 0xff;
				x++;
			}
	}
	lcd_cs_1;
}



/*********************************************************************************************
Function name   : LM6029ACW_PowerSave
Author 					: Grant Phillips
Date Modified   : 25/08/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Puts the lcd in power save mode; display off and backlight off (if BLA pin is
									#defined)

Special Note(s) : NONE

Parameters			: NONE
Return value		: NONE
*********************************************************************************************/
void LM6029ACW_PowerSave(void)
{
#ifdef lcd_bla_1
	lcd_bla_0;
#endif
	lcd_cs_0;
	LM6029ACW_Command(0xae);
	LM6029ACW_Command(0xa5);
	lcd_cs_1;
}



/*********************************************************************************************
Function name   : LM6029ACW_PowerActive
Author 					: Grant Phillips
Date Modified   : 25/08/2013
Compiler        : ARM GCC, Keil ARM-MDK

Description			: Puts the lcd in normal power mode; display on and backlight on (if BLA pin
									is #defined)

Special Note(s) : NONE

Parameters			: NONE
Return value		: NONE
*********************************************************************************************/
void LM6029ACW_PowerActive(void)
{
#ifdef lcd_bla_1
	lcd_bla_1;
#endif
	lcd_cs_0;
	LM6029ACW_Command(0xa4);
	LM6029ACW_Command(0xaf);
	lcd_cs_1;
}



/*********************************************************************************************
Function name   : LM6029ACW_DrawPixel
Author 					: Grant Phillips
Date Modified   : 25/08/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Draws a pixel at the specified xy position

Special Note(s) : NONE

Parameters			: x				-	x position (0 - 127)
									y				-	y position (0 - 63)
									color		-	color of the pixel; cleared(0), filled(1), or toggle(2)
Return value		: NONE
*********************************************************************************************/
void LM6029ACW_DrawPixel(unsigned char x, unsigned char y, unsigned char color)
{
	unsigned char page, temp1, temp2;

	page = y >> 3;
	y = y & 0x07;
	temp1 = LM6029ACW_DisplayRAM[x][page];
	temp2 = 1 << y;
	switch(color)
	{
		case 0:	temp1 = temp1 & ~temp2;
      			break;
		case 1: temp1 = temp1 | temp2;
				break;
		case 2: temp1 = temp1 ^ temp2;
				break;
	}
	LM6029ACW_DisplayRAM[x][page] = temp1;
	lcd_cs_0;
	LM6029ACW_SetColumn(x);
	LM6029ACW_SetPage(page);
	LM6029ACW_WriteData((unsigned char)(temp1));
	lcd_cs_1;
}



/*********************************************************************************************
Function name   : LM6029ACW_DrawLine
Author 					: Grant Phillips
Date Modified   : 25/08/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Draws a line from x1,y1 to x2,y2 in color

Special Note(s) : NONE

Parameters			: x1			-	x1 position (0 - 127)
									y1			-	y1 position (0 - 63)
									x2			-	x2 position (0 - 127)
									y2			-	y2 position (0 - 63)
									color		-	color of the pixel; cleared(0), filled(1), or toggle(2)
Return value		: NONE
*********************************************************************************************/
void LM6029ACW_DrawLine(int x1, int y1, int x2, int y2, unsigned int color)
{
   int        dy, dx;
   signed char  addx=1, addy=1;
   signed int P, diff;

   int i=0;
   dx = fabs((signed int)(x2 - x1));
   dy = fabs((signed int)(y2 - y1));

   if(x1 > x2)
      addx = -1;
   if(y1 > y2)
      addy = -1;

   if(dx >= dy)
   {
      dy *= 2;
      P = dy - dx;
      diff = P - dx;

      for(; i<=dx; ++i)
      {
         LM6029ACW_DrawPixel(x1, y1, color);

         if(P < 0)
         {
            P  += dy;
            x1 += addx;
         }
         else
         {
            P  += diff;
            x1 += addx;
            y1 += addy;
         }
      }
   }
   else
   {
      dx *= 2;
      P = dx - dy;
      diff = P - dy;

      for(; i<=dy; ++i)
      {
    	  LM6029ACW_DrawPixel(x1, y1, color);

         if(P < 0)
         {
            P  += dx;
            y1 += addy;
         }
         else
         {
            P  += diff;
            x1 += addx;
            y1 += addy;
         }
      }
   }
}



/*********************************************************************************************
Function name   : LM6029ACW_DrawCircle
Author 					: Grant Phillips
Date Modified   : 25/08/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Draws a circle at x,y.  The color can set and the filled or not filled 
									option can be selected.

Special Note(s) : NONE

Parameters			: x				-	x position (0 - 127)
									y				-	y position (0 - 63)
									radius	-	radius of the circle (0 - 127)
									fill		-	circle is filled (=1) or not filled (=0)
									color		-	color of the pixel; cleared(0), filled(1)
Return value		: NONE
*********************************************************************************************/
void LM6029ACW_DrawCircle(unsigned char x, unsigned char y, unsigned char radius, unsigned char fill, unsigned char color)
{
   signed char a, b, P;

   a = 0;
   b = radius;
   P = 1 - radius;

   do
   {
      if(fill)
      {
         LM6029ACW_DrawLine(x-a, y+b, x+a, y+b, color);
         LM6029ACW_DrawLine(x-a, y-b, x+a, y-b, color);
         LM6029ACW_DrawLine(x-b, y+a, x+b, y+a, color);
         LM6029ACW_DrawLine(x-b, y-a, x+b, y-a, color);
      }
      else
      {
         LM6029ACW_DrawPixel(a+x, b+y, color);
         LM6029ACW_DrawPixel(b+x, a+y, color);
         LM6029ACW_DrawPixel(x-a, b+y, color);
         LM6029ACW_DrawPixel(x-b, a+y, color);
         LM6029ACW_DrawPixel(b+x, y-a, color);
         LM6029ACW_DrawPixel(a+x, y-b, color);
         LM6029ACW_DrawPixel(x-a, y-b, color);
         LM6029ACW_DrawPixel(x-b, y-a, color);
      }

      if(P < 0)
         P += 3 + 2 * a++;
      else
         P += 5 + 2 * (a++ - b--);
    } while(a <= b);
}



/*********************************************************************************************
Function name   : LM6029ACW_GotoXY
Author 					: Grant Phillips
Date Modified   : 25/08/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Changes the position of the current character cursor from where the next 
									characters will be printed.

Special Note(s) : NONE

Parameters			: x				-	column position (0 - 20)
									y				-	row position (0 - 7)

Return value		: NONE
*********************************************************************************************/
void LM6029ACW_GotoXY(unsigned char x, unsigned char y)
{
	//if((x>=0)&&(x<=20))
  if(x<=20)
		lcd_char_x = x;
	else
		lcd_char_x = 0;
	//if((y>=0)&&(y<=7))
	if(y<=7)
		lcd_char_y = y;
	else
		lcd_char_y = 0;
}



/*********************************************************************************************
Function name   : LM6029ACW_PutStr
Author 					: Grant Phillips
Date Modified   : 25/08/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Prints a string at the current character cursor position with a selected
									color.

Special Note(s) : NONE

Parameters			: str			-	string (char array) to print
									color		-	color of the char in the string; normal(1), inverted(0)

Return value		: NONE
*********************************************************************************************/
void LM6029ACW_PutStr(char *str, unsigned char color)
{
	unsigned int i=0;

	do
	{
		LM6029ACW_PutChar(str[i], color);
		i++;
	}while(str[i]!='\0');

}



/*********************************************************************************************
Function name   : LM6029ACW_PutChar
Author 					: Grant Phillips
Date Modified   : 25/08/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Prints a character at the current character cursor position with a selected
									color.

Special Note(s) : NONE

Parameters			: c				-	character to print
									color		-	color of the character; normal(1), inverted(0)

Return value		: NONE
*********************************************************************************************/
void LM6029ACW_PutChar(unsigned char c, unsigned char color)
{
	unsigned char x, page;

	x = lcd_char_x * 6 + 1;
	page = lcd_char_y;

	LM6029ACW_DrawChar(c, x, page, color);
	if(lcd_char_x == 20)
	{
		lcd_char_x=0;
		if(lcd_char_y==7)
			lcd_char_y=0;
		else
			lcd_char_y++;
	}
	else
		lcd_char_x++;
}


/*********************************************************************************************
Function name   : LM6029ACW_WriteCharCol
Author 					: Grant Phillips
Date Modified   : 25/08/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Used by LM6029ACW_DrawChar only updates the column and page numbers.
*********************************************************************************************/
void LM6029ACW_WriteCharCol(unsigned char v, unsigned char x, unsigned char page, unsigned char color)
{
	if(color==0)
	{
		LM6029ACW_WriteData(~v);
		LM6029ACW_DisplayRAM[x][page] = ~v;
	}
	else
	{
		LM6029ACW_WriteData(v);
		LM6029ACW_DisplayRAM[x][page] = v;
	}
}



/*********************************************************************************************
Function name   : LM6029ACW_DrawChar
Author 					: Grant Phillips
Date Modified   : 25/08/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Used by LM6029ACW_PutChar() only and draws the specified character at byte 
									level using a set font.
*********************************************************************************************/
void LM6029ACW_DrawChar(unsigned char c, unsigned char x, unsigned char page, unsigned char color)
{
	lcd_cs_0;
	LM6029ACW_SetColumn(x);
	LM6029ACW_SetPage(page);

	switch(c)
	{
		case 32:	// 'SPACE'
		{
			LM6029ACW_WriteCharCol(0x00, x, page, color);
			LM6029ACW_WriteCharCol(0x00, x+1, page, color);
			LM6029ACW_WriteCharCol(0x00, x+2, page, color);
			LM6029ACW_WriteCharCol(0x00, x+3, page, color);
			LM6029ACW_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 33:	// '!'
		{
			LM6029ACW_WriteCharCol(0x00, x, page, color);
			LM6029ACW_WriteCharCol(0x00, x+1, page, color);
			LM6029ACW_WriteCharCol(0x9e, x+2, page, color);
			LM6029ACW_WriteCharCol(0x00, x+3, page, color);
			LM6029ACW_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 34:	// '"'
		{
			LM6029ACW_WriteCharCol(0x00, x, page, color);
			LM6029ACW_WriteCharCol(0x0e, x+1, page, color);
			LM6029ACW_WriteCharCol(0x00, x+2, page, color);
			LM6029ACW_WriteCharCol(0x0e, x+3, page, color);
			LM6029ACW_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 35:	// '#'
		{
			LM6029ACW_WriteCharCol(0x28, x, page, color);
			LM6029ACW_WriteCharCol(0xfe, x+1, page, color);
			LM6029ACW_WriteCharCol(0x28, x+2, page, color);
			LM6029ACW_WriteCharCol(0xfe, x+3, page, color);
			LM6029ACW_WriteCharCol(0x28, x+4, page, color);
		}break;

		case 36:	// '$'
		{
			LM6029ACW_WriteCharCol(0x48, x, page, color);
			LM6029ACW_WriteCharCol(0x54, x+1, page, color);
			LM6029ACW_WriteCharCol(0xfe, x+2, page, color);
			LM6029ACW_WriteCharCol(0x54, x+3, page, color);
			LM6029ACW_WriteCharCol(0x24, x+4, page, color);
		}break;

		case 37:	// '%'
		{
			LM6029ACW_WriteCharCol(0x46, x, page, color);
			LM6029ACW_WriteCharCol(0x26, x+1, page, color);
			LM6029ACW_WriteCharCol(0x10, x+2, page, color);
			LM6029ACW_WriteCharCol(0xc8, x+3, page, color);
			LM6029ACW_WriteCharCol(0xc4, x+4, page, color);
		}break;

		case 38:	// '&'
		{
			LM6029ACW_WriteCharCol(0x6c, x, page, color);
			LM6029ACW_WriteCharCol(0x92, x+1, page, color);
			LM6029ACW_WriteCharCol(0xaa, x+2, page, color);
			LM6029ACW_WriteCharCol(0x44, x+3, page, color);
			LM6029ACW_WriteCharCol(0xa0, x+4, page, color);
		}break;

		case 39:	// '''
		{
			LM6029ACW_WriteCharCol(0x00, x, page, color);
			LM6029ACW_WriteCharCol(0x0a, x+1, page, color);
			LM6029ACW_WriteCharCol(0x06, x+2, page, color);
			LM6029ACW_WriteCharCol(0x00, x+3, page, color);
			LM6029ACW_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 40:	// '('
		{
			LM6029ACW_WriteCharCol(0x00, x, page, color);
			LM6029ACW_WriteCharCol(0x38, x+1, page, color);
			LM6029ACW_WriteCharCol(0x44, x+2, page, color);
			LM6029ACW_WriteCharCol(0x82, x+3, page, color);
			LM6029ACW_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 41:	// ')'
		{
			LM6029ACW_WriteCharCol(0x00, x, page, color);
			LM6029ACW_WriteCharCol(0x82, x+1, page, color);
			LM6029ACW_WriteCharCol(0x44, x+2, page, color);
			LM6029ACW_WriteCharCol(0x38, x+3, page, color);
			LM6029ACW_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 42:	// '*'
		{
			LM6029ACW_WriteCharCol(0x28, x, page, color);
			LM6029ACW_WriteCharCol(0x10, x+1, page, color);
			LM6029ACW_WriteCharCol(0x7c, x+2, page, color);
			LM6029ACW_WriteCharCol(0x10, x+3, page, color);
			LM6029ACW_WriteCharCol(0x28, x+4, page, color);
		}break;

		case 43:	// '+'
		{
			LM6029ACW_WriteCharCol(0x10, x, page, color);
			LM6029ACW_WriteCharCol(0x10, x+1, page, color);
			LM6029ACW_WriteCharCol(0x7c, x+2, page, color);
			LM6029ACW_WriteCharCol(0x10, x+3, page, color);
			LM6029ACW_WriteCharCol(0x10, x+4, page, color);
		}break;

		case 44:	// ','
		{
			LM6029ACW_WriteCharCol(0x00, x, page, color);
			LM6029ACW_WriteCharCol(0xa0, x+1, page, color);
			LM6029ACW_WriteCharCol(0x60, x+2, page, color);
			LM6029ACW_WriteCharCol(0x00, x+3, page, color);
			LM6029ACW_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 45:	// '-'
		{
			LM6029ACW_WriteCharCol(0x10, x, page, color);
			LM6029ACW_WriteCharCol(0x10, x+1, page, color);
			LM6029ACW_WriteCharCol(0x10, x+2, page, color);
			LM6029ACW_WriteCharCol(0x10, x+3, page, color);
			LM6029ACW_WriteCharCol(0x10, x+4, page, color);
		}break;

		case 46:	// '.'
		{
			LM6029ACW_WriteCharCol(0x00, x, page, color);
			LM6029ACW_WriteCharCol(0xc0, x+1, page, color);
			LM6029ACW_WriteCharCol(0xc0, x+2, page, color);
			LM6029ACW_WriteCharCol(0x00, x+3, page, color);
			LM6029ACW_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 47:	// '/'
		{
			LM6029ACW_WriteCharCol(0x40, x, page, color);
			LM6029ACW_WriteCharCol(0x20, x+1, page, color);
			LM6029ACW_WriteCharCol(0x10, x+2, page, color);
			LM6029ACW_WriteCharCol(0x08, x+3, page, color);
			LM6029ACW_WriteCharCol(0x04, x+4, page, color);
		}break;

		case 48:	// '0'
		{
			LM6029ACW_WriteCharCol(0x7c, x, page, color);
			LM6029ACW_WriteCharCol(0xa2, x+1, page, color);
			LM6029ACW_WriteCharCol(0x92, x+2, page, color);
			LM6029ACW_WriteCharCol(0x8a, x+3, page, color);
			LM6029ACW_WriteCharCol(0x7c, x+4, page, color);
		}break;

		case 49:	// '1'
		{
			LM6029ACW_WriteCharCol(0x00, x, page, color);
			LM6029ACW_WriteCharCol(0x84, x+1, page, color);
			LM6029ACW_WriteCharCol(0xfe, x+2, page, color);
			LM6029ACW_WriteCharCol(0x80, x+3, page, color);
			LM6029ACW_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 50:	// '2'
		{
			LM6029ACW_WriteCharCol(0x84, x, page, color);
			LM6029ACW_WriteCharCol(0xc2, x+1, page, color);
			LM6029ACW_WriteCharCol(0xa2, x+2, page, color);
			LM6029ACW_WriteCharCol(0x92, x+3, page, color);
			LM6029ACW_WriteCharCol(0x8c, x+4, page, color);
		}break;

		case 51:	// '3'
		{
			LM6029ACW_WriteCharCol(0x42, x, page, color);
			LM6029ACW_WriteCharCol(0x82, x+1, page, color);
			LM6029ACW_WriteCharCol(0x8a, x+2, page, color);
			LM6029ACW_WriteCharCol(0x96, x+3, page, color);
			LM6029ACW_WriteCharCol(0x62, x+4, page, color);
		}break;

		case 52:	// '4'
		{
			LM6029ACW_WriteCharCol(0x30, x, page, color);
			LM6029ACW_WriteCharCol(0x28, x+1, page, color);
			LM6029ACW_WriteCharCol(0x24, x+2, page, color);
			LM6029ACW_WriteCharCol(0xfe, x+3, page, color);
			LM6029ACW_WriteCharCol(0x20, x+4, page, color);
		}break;

		case 53:	// '5'
		{
			LM6029ACW_WriteCharCol(0x4e, x, page, color);
			LM6029ACW_WriteCharCol(0x8a, x+1, page, color);
			LM6029ACW_WriteCharCol(0x8a, x+2, page, color);
			LM6029ACW_WriteCharCol(0x8a, x+3, page, color);
			LM6029ACW_WriteCharCol(0x72, x+4, page, color);
		}break;

		case 54:	// '6'
		{
			LM6029ACW_WriteCharCol(0x78, x, page, color);
			LM6029ACW_WriteCharCol(0x94, x+1, page, color);
			LM6029ACW_WriteCharCol(0x92, x+2, page, color);
			LM6029ACW_WriteCharCol(0x92, x+3, page, color);
			LM6029ACW_WriteCharCol(0x60, x+4, page, color);
		}break;

		case 55:	// '7'
		{
			LM6029ACW_WriteCharCol(0x02, x, page, color);
			LM6029ACW_WriteCharCol(0xe2, x+1, page, color);
			LM6029ACW_WriteCharCol(0x12, x+2, page, color);
			LM6029ACW_WriteCharCol(0x0a, x+3, page, color);
			LM6029ACW_WriteCharCol(0x06, x+4, page, color);
		}break;

		case 56:	// '8'
		{
			LM6029ACW_WriteCharCol(0x6c, x, page, color);
			LM6029ACW_WriteCharCol(0x92, x+1, page, color);
			LM6029ACW_WriteCharCol(0x92, x+2, page, color);
			LM6029ACW_WriteCharCol(0x92, x+3, page, color);
			LM6029ACW_WriteCharCol(0x6c, x+4, page, color);
		}break;

		case 57:	// '9'
		{
			LM6029ACW_WriteCharCol(0x0c, x, page, color);
			LM6029ACW_WriteCharCol(0x92, x+1, page, color);
			LM6029ACW_WriteCharCol(0x92, x+2, page, color);
			LM6029ACW_WriteCharCol(0x52, x+3, page, color);
			LM6029ACW_WriteCharCol(0x3c, x+4, page, color);
		}break;

		case 58:	// ':'
		{
			LM6029ACW_WriteCharCol(0x00, x, page, color);
			LM6029ACW_WriteCharCol(0x6c, x+1, page, color);
			LM6029ACW_WriteCharCol(0x6c, x+2, page, color);
			LM6029ACW_WriteCharCol(0x00, x+3, page, color);
			LM6029ACW_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 59:	// ';'
		{
			LM6029ACW_WriteCharCol(0x00, x, page, color);
			LM6029ACW_WriteCharCol(0xac, x+1, page, color);
			LM6029ACW_WriteCharCol(0x6c, x+2, page, color);
			LM6029ACW_WriteCharCol(0x00, x+3, page, color);
			LM6029ACW_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 60:	// '<'
		{
			LM6029ACW_WriteCharCol(0x10, x, page, color);
			LM6029ACW_WriteCharCol(0x28, x+1, page, color);
			LM6029ACW_WriteCharCol(0x44, x+2, page, color);
			LM6029ACW_WriteCharCol(0x82, x+3, page, color);
			LM6029ACW_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 61:	// '='
		{
			LM6029ACW_WriteCharCol(0x28, x, page, color);
			LM6029ACW_WriteCharCol(0x28, x+1, page, color);
			LM6029ACW_WriteCharCol(0x28, x+2, page, color);
			LM6029ACW_WriteCharCol(0x28, x+3, page, color);
			LM6029ACW_WriteCharCol(0x28, x+4, page, color);
		}break;

		case 62:	// '>'
		{
			LM6029ACW_WriteCharCol(0x00, x, page, color);
			LM6029ACW_WriteCharCol(0x82, x+1, page, color);
			LM6029ACW_WriteCharCol(0x44, x+2, page, color);
			LM6029ACW_WriteCharCol(0x28, x+3, page, color);
			LM6029ACW_WriteCharCol(0x10, x+4, page, color);
		}break;

		case 63:	// '?'
		{
			LM6029ACW_WriteCharCol(0x04, x, page, color);
			LM6029ACW_WriteCharCol(0x02, x+1, page, color);
			LM6029ACW_WriteCharCol(0xa2, x+2, page, color);
			LM6029ACW_WriteCharCol(0x12, x+3, page, color);
			LM6029ACW_WriteCharCol(0x0c, x+4, page, color);
		}break;

		case 64:	// '@'
		{
			LM6029ACW_WriteCharCol(0x64, x, page, color);
			LM6029ACW_WriteCharCol(0x92, x+1, page, color);
			LM6029ACW_WriteCharCol(0xf2, x+2, page, color);
			LM6029ACW_WriteCharCol(0x82, x+3, page, color);
			LM6029ACW_WriteCharCol(0x7c, x+4, page, color);
		}break;

		case 65:	// 'A'
		{
			LM6029ACW_WriteCharCol(0xfc, x, page, color);
			LM6029ACW_WriteCharCol(0x22, x+1, page, color);
			LM6029ACW_WriteCharCol(0x22, x+2, page, color);
			LM6029ACW_WriteCharCol(0x22, x+3, page, color);
			LM6029ACW_WriteCharCol(0xfc, x+4, page, color);
		}break;

		case 66:	// 'B'
		{
			LM6029ACW_WriteCharCol(0xfe, x, page, color);
			LM6029ACW_WriteCharCol(0x92, x+1, page, color);
			LM6029ACW_WriteCharCol(0x92, x+2, page, color);
			LM6029ACW_WriteCharCol(0x92, x+3, page, color);
			LM6029ACW_WriteCharCol(0x6c, x+4, page, color);
		}break;

		case 67:	// 'C'
		{
			LM6029ACW_WriteCharCol(0x7c, x, page, color);
			LM6029ACW_WriteCharCol(0x82, x+1, page, color);
			LM6029ACW_WriteCharCol(0x82, x+2, page, color);
			LM6029ACW_WriteCharCol(0x82, x+3, page, color);
			LM6029ACW_WriteCharCol(0x44, x+4, page, color);
		}break;

		case 68:	// 'D'
		{
			LM6029ACW_WriteCharCol(0xfe, x, page, color);
			LM6029ACW_WriteCharCol(0x82, x+1, page, color);
			LM6029ACW_WriteCharCol(0x82, x+2, page, color);
			LM6029ACW_WriteCharCol(0x44, x+3, page, color);
			LM6029ACW_WriteCharCol(0x38, x+4, page, color);
		}break;

		case 69:	// 'E'
		{
			LM6029ACW_WriteCharCol(0xfe, x, page, color);
			LM6029ACW_WriteCharCol(0x92, x+1, page, color);
			LM6029ACW_WriteCharCol(0x92, x+2, page, color);
			LM6029ACW_WriteCharCol(0x92, x+3, page, color);
			LM6029ACW_WriteCharCol(0x82, x+4, page, color);
		}break;

		case 70:	// 'F'
		{
			LM6029ACW_WriteCharCol(0xfe, x, page, color);
			LM6029ACW_WriteCharCol(0x12, x+1, page, color);
			LM6029ACW_WriteCharCol(0x12, x+2, page, color);
			LM6029ACW_WriteCharCol(0x12, x+3, page, color);
			LM6029ACW_WriteCharCol(0x02, x+4, page, color);
		}break;

		case 71:	// 'G'
		{
			LM6029ACW_WriteCharCol(0x7c, x, page, color);
			LM6029ACW_WriteCharCol(0x82, x+1, page, color);
			LM6029ACW_WriteCharCol(0x92, x+2, page, color);
			LM6029ACW_WriteCharCol(0x92, x+3, page, color);
			LM6029ACW_WriteCharCol(0xf4, x+4, page, color);
		}break;

		case 72:	// 'H'
		{
			LM6029ACW_WriteCharCol(0xfe, x, page, color);
			LM6029ACW_WriteCharCol(0x10, x+1, page, color);
			LM6029ACW_WriteCharCol(0x10, x+2, page, color);
			LM6029ACW_WriteCharCol(0x10, x+3, page, color);
			LM6029ACW_WriteCharCol(0xfe, x+4, page, color);
		}break;

		case 73:	// 'I'
		{
			LM6029ACW_WriteCharCol(0x00, x, page, color);
			LM6029ACW_WriteCharCol(0x82, x+1, page, color);
			LM6029ACW_WriteCharCol(0xfe, x+2, page, color);
			LM6029ACW_WriteCharCol(0x82, x+3, page, color);
			LM6029ACW_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 74:	// 'J'
		{
			LM6029ACW_WriteCharCol(0x40, x, page, color);
			LM6029ACW_WriteCharCol(0x80, x+1, page, color);
			LM6029ACW_WriteCharCol(0x82, x+2, page, color);
			LM6029ACW_WriteCharCol(0x7e, x+3, page, color);
			LM6029ACW_WriteCharCol(0x02, x+4, page, color);
		}break;

		case 75:	// 'K'
		{
			LM6029ACW_WriteCharCol(0xfe, x, page, color);
			LM6029ACW_WriteCharCol(0x10, x+1, page, color);
			LM6029ACW_WriteCharCol(0x28, x+2, page, color);
			LM6029ACW_WriteCharCol(0x44, x+3, page, color);
			LM6029ACW_WriteCharCol(0x82, x+4, page, color);
		}break;

		case 76:	// 'L'
		{
			LM6029ACW_WriteCharCol(0xfe, x, page, color);
			LM6029ACW_WriteCharCol(0x80, x+1, page, color);
			LM6029ACW_WriteCharCol(0x80, x+2, page, color);
			LM6029ACW_WriteCharCol(0x80, x+3, page, color);
			LM6029ACW_WriteCharCol(0x80, x+4, page, color);
		}break;

		case 77:	// 'M'
		{
			LM6029ACW_WriteCharCol(0xfe, x, page, color);
			LM6029ACW_WriteCharCol(0x04, x+1, page, color);
			LM6029ACW_WriteCharCol(0x18, x+2, page, color);
			LM6029ACW_WriteCharCol(0x04, x+3, page, color);
			LM6029ACW_WriteCharCol(0xfe, x+4, page, color);
		}break;

		case 78:	// 'N'
		{
			LM6029ACW_WriteCharCol(0xfe, x, page, color);
			LM6029ACW_WriteCharCol(0x08, x+1, page, color);
			LM6029ACW_WriteCharCol(0x10, x+2, page, color);
			LM6029ACW_WriteCharCol(0x20, x+3, page, color);
			LM6029ACW_WriteCharCol(0xfe, x+4, page, color);
		}break;

		case 79:	// 'O'
		{
			LM6029ACW_WriteCharCol(0x7c, x, page, color);
			LM6029ACW_WriteCharCol(0x82, x+1, page, color);
			LM6029ACW_WriteCharCol(0x82, x+2, page, color);
			LM6029ACW_WriteCharCol(0x82, x+3, page, color);
			LM6029ACW_WriteCharCol(0x7c, x+4, page, color);
		}break;

		case 80:	// 'P'
		{
			LM6029ACW_WriteCharCol(0xfe, x, page, color);
			LM6029ACW_WriteCharCol(0x12, x+1, page, color);
			LM6029ACW_WriteCharCol(0x12, x+2, page, color);
			LM6029ACW_WriteCharCol(0x12, x+3, page, color);
			LM6029ACW_WriteCharCol(0x0c, x+4, page, color);
		}break;

		case 81:	// 'Q'
		{
			LM6029ACW_WriteCharCol(0x7c, x, page, color);
			LM6029ACW_WriteCharCol(0x82, x+1, page, color);
			LM6029ACW_WriteCharCol(0xa2, x+2, page, color);
			LM6029ACW_WriteCharCol(0x42, x+3, page, color);
			LM6029ACW_WriteCharCol(0xbc, x+4, page, color);
		}break;

		case 82:	// 'R'
		{
			LM6029ACW_WriteCharCol(0xfe, x, page, color);
			LM6029ACW_WriteCharCol(0x12, x+1, page, color);
			LM6029ACW_WriteCharCol(0x32, x+2, page, color);
			LM6029ACW_WriteCharCol(0x52, x+3, page, color);
			LM6029ACW_WriteCharCol(0x8c, x+4, page, color);
		}break;

		case 83:	// 'S'
		{
			LM6029ACW_WriteCharCol(0x8c, x, page, color);
			LM6029ACW_WriteCharCol(0x92, x+1, page, color);
			LM6029ACW_WriteCharCol(0x92, x+2, page, color);
			LM6029ACW_WriteCharCol(0x92, x+3, page, color);
			LM6029ACW_WriteCharCol(0x62, x+4, page, color);
		}break;

		case 84:	// 'T'
		{
			LM6029ACW_WriteCharCol(0x02, x, page, color);
			LM6029ACW_WriteCharCol(0x02, x+1, page, color);
			LM6029ACW_WriteCharCol(0xfe, x+2, page, color);
			LM6029ACW_WriteCharCol(0x02, x+3, page, color);
			LM6029ACW_WriteCharCol(0x02, x+4, page, color);
		}break;

		case 85:	// 'U'
		{
			LM6029ACW_WriteCharCol(0x7e, x, page, color);
			LM6029ACW_WriteCharCol(0x80, x+1, page, color);
			LM6029ACW_WriteCharCol(0x80, x+2, page, color);
			LM6029ACW_WriteCharCol(0x80, x+3, page, color);
			LM6029ACW_WriteCharCol(0x7e, x+4, page, color);
		}break;

		case 86:	// 'V'
		{
			LM6029ACW_WriteCharCol(0x3e, x, page, color);
			LM6029ACW_WriteCharCol(0x40, x+1, page, color);
			LM6029ACW_WriteCharCol(0x80, x+2, page, color);
			LM6029ACW_WriteCharCol(0x40, x+3, page, color);
			LM6029ACW_WriteCharCol(0x3e, x+4, page, color);
		}break;

		case 87:	// 'W'
		{
			LM6029ACW_WriteCharCol(0x7e, x, page, color);
			LM6029ACW_WriteCharCol(0x80, x+1, page, color);
			LM6029ACW_WriteCharCol(0x70, x+2, page, color);
			LM6029ACW_WriteCharCol(0x80, x+3, page, color);
			LM6029ACW_WriteCharCol(0x7e, x+4, page, color);
		}break;

		case 88:	// 'X'
		{
			LM6029ACW_WriteCharCol(0xc6, x, page, color);
			LM6029ACW_WriteCharCol(0x28, x+1, page, color);
			LM6029ACW_WriteCharCol(0x10, x+2, page, color);
			LM6029ACW_WriteCharCol(0x28, x+3, page, color);
			LM6029ACW_WriteCharCol(0xc6, x+4, page, color);
		}break;

		case 89:	// 'Y'
		{
			LM6029ACW_WriteCharCol(0x0e, x, page, color);
			LM6029ACW_WriteCharCol(0x10, x+1, page, color);
			LM6029ACW_WriteCharCol(0xe0, x+2, page, color);
			LM6029ACW_WriteCharCol(0x10, x+3, page, color);
			LM6029ACW_WriteCharCol(0x0e, x+4, page, color);
		}break;

		case 90:	// 'Z'
		{
			LM6029ACW_WriteCharCol(0xc2, x, page, color);
			LM6029ACW_WriteCharCol(0xa2, x+1, page, color);
			LM6029ACW_WriteCharCol(0x92, x+2, page, color);
			LM6029ACW_WriteCharCol(0x8a, x+3, page, color);
			LM6029ACW_WriteCharCol(0x86, x+4, page, color);
		}break;

		case 91:	// '['
		{
			LM6029ACW_WriteCharCol(0x00, x, page, color);
			LM6029ACW_WriteCharCol(0xfe, x+1, page, color);
			LM6029ACW_WriteCharCol(0x82, x+2, page, color);
			LM6029ACW_WriteCharCol(0x82, x+3, page, color);
			LM6029ACW_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 92:	// '\'
		{
			LM6029ACW_WriteCharCol(0x04, x, page, color);
			LM6029ACW_WriteCharCol(0x08, x+1, page, color);
			LM6029ACW_WriteCharCol(0x10, x+2, page, color);
			LM6029ACW_WriteCharCol(0x20, x+3, page, color);
			LM6029ACW_WriteCharCol(0x40, x+4, page, color);
		}break;

		case 93:	// ']'
		{
			LM6029ACW_WriteCharCol(0x00, x, page, color);
			LM6029ACW_WriteCharCol(0x82, x+1, page, color);
			LM6029ACW_WriteCharCol(0x82, x+2, page, color);
			LM6029ACW_WriteCharCol(0xfe, x+3, page, color);
			LM6029ACW_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 94:	// '^'
		{
			LM6029ACW_WriteCharCol(0x08, x, page, color);
			LM6029ACW_WriteCharCol(0x04, x+1, page, color);
			LM6029ACW_WriteCharCol(0x02, x+2, page, color);
			LM6029ACW_WriteCharCol(0x04, x+3, page, color);
			LM6029ACW_WriteCharCol(0x08, x+4, page, color);
		}break;

		case 95:	// '_'
		{
			LM6029ACW_WriteCharCol(0x80, x, page, color);
			LM6029ACW_WriteCharCol(0x80, x+1, page, color);
			LM6029ACW_WriteCharCol(0x80, x+2, page, color);
			LM6029ACW_WriteCharCol(0x80, x+3, page, color);
			LM6029ACW_WriteCharCol(0x80, x+4, page, color);
		}break;

		case 96:	// '`'
		{
			LM6029ACW_WriteCharCol(0x00, x, page, color);
			LM6029ACW_WriteCharCol(0x02, x+1, page, color);
			LM6029ACW_WriteCharCol(0x04, x+2, page, color);
			LM6029ACW_WriteCharCol(0x08, x+3, page, color);
			LM6029ACW_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 97:	// 'a'
		{
			LM6029ACW_WriteCharCol(0x40, x, page, color);
			LM6029ACW_WriteCharCol(0xa8, x+1, page, color);
			LM6029ACW_WriteCharCol(0xa8, x+2, page, color);
			LM6029ACW_WriteCharCol(0xa8, x+3, page, color);
			LM6029ACW_WriteCharCol(0xf0, x+4, page, color);
		}break;

		case 98:	// 'b'
		{
			LM6029ACW_WriteCharCol(0xfe, x, page, color);
			LM6029ACW_WriteCharCol(0x90, x+1, page, color);
			LM6029ACW_WriteCharCol(0x88, x+2, page, color);
			LM6029ACW_WriteCharCol(0x88, x+3, page, color);
			LM6029ACW_WriteCharCol(0x70, x+4, page, color);
		}break;

		case 99:	// 'c'
		{
			LM6029ACW_WriteCharCol(0x70, x, page, color);
			LM6029ACW_WriteCharCol(0x88, x+1, page, color);
			LM6029ACW_WriteCharCol(0x88, x+2, page, color);
			LM6029ACW_WriteCharCol(0x88, x+3, page, color);
			LM6029ACW_WriteCharCol(0x40, x+4, page, color);
		}break;

		case 100:	// 'd'
		{
			LM6029ACW_WriteCharCol(0x70, x, page, color);
			LM6029ACW_WriteCharCol(0x88, x+1, page, color);
			LM6029ACW_WriteCharCol(0x88, x+2, page, color);
			LM6029ACW_WriteCharCol(0x90, x+3, page, color);
			LM6029ACW_WriteCharCol(0xfe, x+4, page, color);
		}break;

		case 101:	// 'e'
		{
			LM6029ACW_WriteCharCol(0x70, x, page, color);
			LM6029ACW_WriteCharCol(0xa8, x+1, page, color);
			LM6029ACW_WriteCharCol(0xa8, x+2, page, color);
			LM6029ACW_WriteCharCol(0xa8, x+3, page, color);
			LM6029ACW_WriteCharCol(0x30, x+4, page, color);
		}break;

		case 102:	// 'f'
		{
			LM6029ACW_WriteCharCol(0x10, x, page, color);
			LM6029ACW_WriteCharCol(0xfc, x+1, page, color);
			LM6029ACW_WriteCharCol(0x12, x+2, page, color);
			LM6029ACW_WriteCharCol(0x02, x+3, page, color);
			LM6029ACW_WriteCharCol(0x04, x+4, page, color);
		}break;

		case 103:	// 'g'
		{
			LM6029ACW_WriteCharCol(0x18, x, page, color);
			LM6029ACW_WriteCharCol(0xa4, x+1, page, color);
			LM6029ACW_WriteCharCol(0xa4, x+2, page, color);
			LM6029ACW_WriteCharCol(0xa4, x+3, page, color);
			LM6029ACW_WriteCharCol(0x7c, x+4, page, color);
		}break;

		case 104:	// 'h'
		{
			LM6029ACW_WriteCharCol(0xfe, x, page, color);
			LM6029ACW_WriteCharCol(0x10, x+1, page, color);
			LM6029ACW_WriteCharCol(0x08, x+2, page, color);
			LM6029ACW_WriteCharCol(0x08, x+3, page, color);
			LM6029ACW_WriteCharCol(0xf0, x+4, page, color);
		}break;

		case 105:	// 'i'
		{
			LM6029ACW_WriteCharCol(0x00, x, page, color);
			LM6029ACW_WriteCharCol(0x88, x+1, page, color);
			LM6029ACW_WriteCharCol(0xfa, x+2, page, color);
			LM6029ACW_WriteCharCol(0x80, x+3, page, color);
			LM6029ACW_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 106:	// 'j'
		{
			LM6029ACW_WriteCharCol(0x40, x, page, color);
			LM6029ACW_WriteCharCol(0x80, x+1, page, color);
			LM6029ACW_WriteCharCol(0x88, x+2, page, color);
			LM6029ACW_WriteCharCol(0x7a, x+3, page, color);
			LM6029ACW_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 107:	// 'k'
		{
			LM6029ACW_WriteCharCol(0xfe, x, page, color);
			LM6029ACW_WriteCharCol(0x20, x+1, page, color);
			LM6029ACW_WriteCharCol(0x50, x+2, page, color);
			LM6029ACW_WriteCharCol(0x88, x+3, page, color);
			LM6029ACW_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 108:	// 'l'
		{
			LM6029ACW_WriteCharCol(0x00, x, page, color);
			LM6029ACW_WriteCharCol(0x82, x+1, page, color);
			LM6029ACW_WriteCharCol(0xfe, x+2, page, color);
			LM6029ACW_WriteCharCol(0x80, x+3, page, color);
			LM6029ACW_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 109:	// 'm'
		{
			LM6029ACW_WriteCharCol(0xf8, x, page, color);
			LM6029ACW_WriteCharCol(0x08, x+1, page, color);
			LM6029ACW_WriteCharCol(0x30, x+2, page, color);
			LM6029ACW_WriteCharCol(0x08, x+3, page, color);
			LM6029ACW_WriteCharCol(0xf0, x+4, page, color);
		}break;

		case 110:	// 'n'
		{
			LM6029ACW_WriteCharCol(0xf8, x, page, color);
			LM6029ACW_WriteCharCol(0x10, x+1, page, color);
			LM6029ACW_WriteCharCol(0x08, x+2, page, color);
			LM6029ACW_WriteCharCol(0x08, x+3, page, color);
			LM6029ACW_WriteCharCol(0xf0, x+4, page, color);
		}break;

		case 111:	// 'o'
		{
			LM6029ACW_WriteCharCol(0x70, x, page, color);
			LM6029ACW_WriteCharCol(0x88, x+1, page, color);
			LM6029ACW_WriteCharCol(0x88, x+2, page, color);
			LM6029ACW_WriteCharCol(0x88, x+3, page, color);
			LM6029ACW_WriteCharCol(0x70, x+4, page, color);
		}break;

		case 112:	// 'p'
		{
			LM6029ACW_WriteCharCol(0xf8, x, page, color);
			LM6029ACW_WriteCharCol(0x28, x+1, page, color);
			LM6029ACW_WriteCharCol(0x28, x+2, page, color);
			LM6029ACW_WriteCharCol(0x28, x+3, page, color);
			LM6029ACW_WriteCharCol(0x10, x+4, page, color);
		}break;

		case 113:	// 'q'
		{
			LM6029ACW_WriteCharCol(0x10, x, page, color);
			LM6029ACW_WriteCharCol(0x28, x+1, page, color);
			LM6029ACW_WriteCharCol(0x28, x+2, page, color);
			LM6029ACW_WriteCharCol(0x30, x+3, page, color);
			LM6029ACW_WriteCharCol(0xf8, x+4, page, color);
		}break;

		case 114:	// 'r'
		{
			LM6029ACW_WriteCharCol(0xf8, x, page, color);
			LM6029ACW_WriteCharCol(0x10, x+1, page, color);
			LM6029ACW_WriteCharCol(0x08, x+2, page, color);
			LM6029ACW_WriteCharCol(0x08, x+3, page, color);
			LM6029ACW_WriteCharCol(0x10, x+4, page, color);
		}break;

		case 115:	// 's'
		{
			LM6029ACW_WriteCharCol(0x90, x, page, color);
			LM6029ACW_WriteCharCol(0xa8, x+1, page, color);
			LM6029ACW_WriteCharCol(0xa8, x+2, page, color);
			LM6029ACW_WriteCharCol(0xa8, x+3, page, color);
			LM6029ACW_WriteCharCol(0x40, x+4, page, color);
		}break;

		case 116:	// 't'
		{
			LM6029ACW_WriteCharCol(0x08, x, page, color);
			LM6029ACW_WriteCharCol(0x7e, x+1, page, color);
			LM6029ACW_WriteCharCol(0x88, x+2, page, color);
			LM6029ACW_WriteCharCol(0x80, x+3, page, color);
			LM6029ACW_WriteCharCol(0x40, x+4, page, color);
		}break;

		case 117:	// 'u'
		{
			LM6029ACW_WriteCharCol(0x78, x, page, color);
			LM6029ACW_WriteCharCol(0x80, x+1, page, color);
			LM6029ACW_WriteCharCol(0x80, x+2, page, color);
			LM6029ACW_WriteCharCol(0x40, x+3, page, color);
			LM6029ACW_WriteCharCol(0xf8, x+4, page, color);
		}break;

		case 118:	// 'v'
		{
			LM6029ACW_WriteCharCol(0x38, x, page, color);
			LM6029ACW_WriteCharCol(0x40, x+1, page, color);
			LM6029ACW_WriteCharCol(0x80, x+2, page, color);
			LM6029ACW_WriteCharCol(0x40, x+3, page, color);
			LM6029ACW_WriteCharCol(0x38, x+4, page, color);
		}break;

		case 119:	// 'w'
		{
			LM6029ACW_WriteCharCol(0x78, x, page, color);
			LM6029ACW_WriteCharCol(0x80, x+1, page, color);
			LM6029ACW_WriteCharCol(0x60, x+2, page, color);
			LM6029ACW_WriteCharCol(0x80, x+3, page, color);
			LM6029ACW_WriteCharCol(0x78, x+4, page, color);
		}break;

		case 120:	// 'x'
		{
			LM6029ACW_WriteCharCol(0x88, x, page, color);
			LM6029ACW_WriteCharCol(0x50, x+1, page, color);
			LM6029ACW_WriteCharCol(0x20, x+2, page, color);
			LM6029ACW_WriteCharCol(0x50, x+3, page, color);
			LM6029ACW_WriteCharCol(0x88, x+4, page, color);
		}break;

		case 121:	// 'y'
		{
			LM6029ACW_WriteCharCol(0x18, x, page, color);
			LM6029ACW_WriteCharCol(0xa0, x+1, page, color);
			LM6029ACW_WriteCharCol(0xa0, x+2, page, color);
			LM6029ACW_WriteCharCol(0xa0, x+3, page, color);
			LM6029ACW_WriteCharCol(0x78, x+4, page, color);
		}break;

		case 122:	// 'z'
		{
			LM6029ACW_WriteCharCol(0x88, x, page, color);
			LM6029ACW_WriteCharCol(0xc8, x+1, page, color);
			LM6029ACW_WriteCharCol(0xa8, x+2, page, color);
			LM6029ACW_WriteCharCol(0x98, x+3, page, color);
			LM6029ACW_WriteCharCol(0x88, x+4, page, color);
		}break;

		case 123:	// '{'
		{
			LM6029ACW_WriteCharCol(0x00, x, page, color);
			LM6029ACW_WriteCharCol(0x10, x+1, page, color);
			LM6029ACW_WriteCharCol(0x6c, x+2, page, color);
			LM6029ACW_WriteCharCol(0x82, x+3, page, color);
			LM6029ACW_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 124:	// '|'
		{
			LM6029ACW_WriteCharCol(0x00, x, page, color);
			LM6029ACW_WriteCharCol(0x00, x+1, page, color);
			LM6029ACW_WriteCharCol(0xfe, x+2, page, color);
			LM6029ACW_WriteCharCol(0x00, x+3, page, color);
			LM6029ACW_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 125:	// '}'
		{
			LM6029ACW_WriteCharCol(0x00, x, page, color);
			LM6029ACW_WriteCharCol(0x82, x+1, page, color);
			LM6029ACW_WriteCharCol(0x6c, x+2, page, color);
			LM6029ACW_WriteCharCol(0x10, x+3, page, color);
			LM6029ACW_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 126:	// '~'
		{
			LM6029ACW_WriteCharCol(0x20, x, page, color);
			LM6029ACW_WriteCharCol(0x10, x+1, page, color);
			LM6029ACW_WriteCharCol(0x10, x+2, page, color);
			LM6029ACW_WriteCharCol(0x20, x+3, page, color);
			LM6029ACW_WriteCharCol(0x10, x+4, page, color);
		}break;
		/*

		case :	// ''
		{
			LM6029ACW_WriteCharCol(0x, x, page, color);
			LM6029ACW_WriteCharCol(0x, x+1, page, color);
			LM6029ACW_WriteCharCol(0x, x+2, page, color);
			LM6029ACW_WriteCharCol(0x, x+3, page, color);
			LM6029ACW_WriteCharCol(0x, x+4, page, color);
		}break;
		*/

		default:	// not supported, print box
		{
			LM6029ACW_WriteData(0xfe);
			LM6029ACW_WriteData(0xfe);
			LM6029ACW_WriteData(0xfe);
			LM6029ACW_WriteData(0xfe);
			LM6029ACW_WriteData(0xfe);
		}
	}
	LM6029ACW_WriteCharCol(0x00, x+5, page, color);	//default last blank column of 6 x 8 character

	lcd_cs_1;
}
